c simulation program for K-nucl production by kaon at rest 
c in liquid helium 4 target.
c the energy of the system assumed to be measured by 
c detecting neutron.
c ... common /RIDS/ reaction ID
c
c All		All with KN	  	:  - (with KN)	p1 : black
c KN		KN reaction  	:  1 (formation)	p1 : black
c tnL		none-mesic Lmbda	:  2 (reaction)	p1 : yellow
c tnS		none-mesic Sigma	:  3 (reaction)	p1 : blue
c pi_abs	pion absorption	:  4 (internal)	p1 : cyan
c SLC		Sigma-Lmbda conv 	:  5 (reaction)	p1 : green
c QFL		QF (mesic) Lmbda	:  6 (reaction)	p1 : magenta
c QFS		QF (mesic) Sigma	:  7 (reaction)	p1 : red
c CBG		const. background	:  8 (accidental)	p1 : blue
c
c -All	All without KN  	:  - (without KN)	p2 : black
c -KN		KN reaction		: -1 (KN decay)	p2 : black
c -tnL	none-mesic Lmbda	: -2 (L decay)	p2 : yellow
c -tnS	none-mesic Sigma	: -3 (S decay)	p2 : blue
c -pI_abs	pion absorption	: -4 (external)	p2 : cyan
c -SLC	igma-Lmbda conv	: -5 (L decay)	p2 : green
c -QFL	QF (mesic) Lmbda	: -6 (L decay)	p2 : magenta
c -QFS	QF (mesic) Sigma	: -7 (S decay)	p2 : red
c -pI_mono	pion abs mono	: -8 (external)	p2 : blue
c
      implicit NONE
c local variables 
      integer j0, j1, j2
	logical KstopF, Trigger
c ... common /setup/ for simulation run-condition, counter resolution and its efficiency
	integer FF_knucl_decay, multi_set, k_decay_set
	logical Kstop_cor
      real  delta_x, delta_y, delta_T0, delta_NC, TC_eff, T0_eff
      common /setup/ delta_x, delta_y, delta_T0, delta_NC, TC_eff, T0_eff,
     &	FF_knucl_decay, multi_set, k_decay_set, Kstop_cor
c common for total kaon number
	integer Knum
	common /KTOT/ Knum
c ... common /DIRS/ for save directory name
	character*80 DirSave
	integer DirL
	common /DIRS/ DirSave, DirL
c common for debug
	logical debug_F
      integer ev_NUM, ev_error
      common /DeBugging/ debug_F, ev_NUM, ev_error
      data  debug_F /.true./
      data  ev_error /2947/
c begin
      call read_quasi_free_branch
	call normalize_branch
      call define_masses
      call define_histogram
      call target_def
      ev_NUM = 0
      write(*,*) 'Please wait. starting event generation.............'
	if(FF_knucl_decay .eq. 1) write(*,*) '### Nuclear Form-Factor !!!'
	if(multi_set .ne. 1) write(*,*) '### Caution NO scattering will be calculated!!!'
	if(k_decay_set .eq. 1) write(*,*) '### Caution kaon will decay!!!'
	if(.not.Kstop_cor) write(*,*) '### Caution T0-time correction will be skiped!!!'
c>>      Open ( unit = 13,
c>>     &     file = DirSave(1:DirL)//'run-monitor.dat',
c>>     &     status = 'unknown' )
c
      do j2 = 1, Knum
	   do j1 =  1, 100
	   do j0 =  1, 1000
            ev_NUM = ev_NUM + 1
            call event_init
            call kaon_beam(KstopF)
		if (KstopF) then
		   call K_reaction
		   call K_trigger(Trigger)
               if (Trigger) call event_analysis
		endif
         enddo
c         write(*,*) 'EV_num=',  ev_NUM
	   enddo
         call branch_print( ev_NUM, j2 )
         call histogram_output
         write(*,*) j2, ' % finished.     EV_number=',  ev_NUM
      enddo
c>>	close (13)
      end




	subroutine PIDdef(b_trg, dL_thick, dL_thin, 
     &	fast_pi_PID, slow_pi_PID, deuteron_PID, proton_PID,
     &	MS_vfast_pi_PID, MS_fast_pi_PID, MS_slow_pi_PID, MS_proton_PID)
	implicit none
c var
	real b_trg, dL_thick, dL_thin, y, x
	logical fast_pi_PID, slow_pi_PID, deuteron_PID, proton_PID
      logical MS_vfast_pi_PID, MS_fast_pi_PID, MS_slow_pi_PID, MS_proton_PID
c external
	logical fast_pi_check, slow_pi_check, deuteron_check, proton_check
	logical MS_fast_pi_check, MS_vfast_pi_check, MS_slow_pi_check, MS_proton_check
c begin
	fast_pi_PID = .false. 
	proton_PID = .false. 
	slow_pi_PID = .false. 
	deuteron_PID = .false. 
      MS_vfast_pi_PID = .false. 
      MS_fast_pi_PID = .false. 
      MS_slow_pi_PID = .false. 
      MS_proton_PID = .false. 
	x = dL_thick
	y = dL_thin
c ... fast_pi_PID
	fast_pi_PID = fast_pi_check(x, y)
	if(fast_pi_PID) go to 10
c ... slow_pi_PID
	slow_pi_PID = slow_pi_check(x, y)
	if(slow_pi_PID) go to 10
c ... proton_PID
	proton_PID = proton_check(x, y)
	if(proton_PID) go to 10
c ... deuteron_PID
	deuteron_PID = deuteron_check(x, y)
c ... MS_PID
10	continue
	MS_fast_pi_PID = MS_fast_pi_check(x, y)
	if(MS_fast_pi_PID) return
	MS_vfast_pi_PID = MS_vfast_pi_check(x, y)
	if(MS_vfast_pi_PID) return
	MS_slow_pi_PID = MS_slow_pi_check(x, y)
	if(MS_slow_pi_PID) return
	MS_proton_PID = MS_proton_check(x, y)
	if(MS_proton_PID) return
	return
	end





	logical function MS_vfast_pi_check(x, y)
      implicit NONE
c input
	real x, y
c local
	logical first /.true./
	integer j
c common for MS_VFPB
	real MS_vfp_boundary(5, 20)
	integer MS_VFPB_N
	common /MS_VFPB/ MS_VFPB_N, MS_vfp_boundary
c external
	logical area_check
c begin
	if(first) then
	   MS_VFPB_N = 4
	   MS_vfp_boundary(1, 1) = 2.
	   MS_vfp_boundary(2, 1) = 0.5
c	   MS_vfp_boundary(1, 2) = 5.5
c	   MS_vfp_boundary(2, 2) = 0.5
	   MS_vfp_boundary(1, 2) = 6.25
	   MS_vfp_boundary(2, 2) = 0.5
c	   MS_vfp_boundary(1, 3) = 4.91667
c	   MS_vfp_boundary(2, 3) = 2.583335
c	   MS_vfp_boundary(1, 3) = 3.66667
c	   MS_vfp_boundary(2, 3) = 2.333335
	   MS_vfp_boundary(1, 3) = 4.29167
	   MS_vfp_boundary(2, 3) = 2.458335
	   MS_vfp_boundary(1, 4) = 2.
	   MS_vfp_boundary(2, 4) = 2.
	   Do j = 1, MS_VFPB_N - 1
		MS_vfp_boundary(3, j) = MS_vfp_boundary(1, j+1) - MS_vfp_boundary(1, j)
		MS_vfp_boundary(4, j) = MS_vfp_boundary(2, j+1) - MS_vfp_boundary(2, j)
		MS_vfp_boundary(5, j) = sqrt( MS_vfp_boundary(3, j)**2 + MS_vfp_boundary(4, j)**2 ) 
		MS_vfp_boundary(3, j) = MS_vfp_boundary(3, j) / MS_vfp_boundary(5, j)
		MS_vfp_boundary(4, j) = MS_vfp_boundary(4, j) / MS_vfp_boundary(5, j)
	   EndDo
	   MS_vfp_boundary(3, MS_VFPB_N) = MS_vfp_boundary(1, 1) - MS_vfp_boundary(1, MS_VFPB_N)
	   MS_vfp_boundary(4, MS_VFPB_N) = MS_vfp_boundary(2, 1) - MS_vfp_boundary(2, MS_VFPB_N)
	   MS_vfp_boundary(5, MS_VFPB_N) = sqrt( MS_vfp_boundary(3, j)**2 + MS_vfp_boundary(4, j)**2 ) 
	   MS_vfp_boundary(3, MS_VFPB_N) = MS_vfp_boundary(3, MS_VFPB_N) / MS_vfp_boundary(5, MS_VFPB_N)
	   MS_vfp_boundary(4, MS_VFPB_N) = MS_vfp_boundary(4, MS_VFPB_N) / MS_vfp_boundary(5, MS_VFPB_N)
	   first = .false.
	endif
	MS_vfast_pi_check = area_check(x, y, MS_vfp_boundary, MS_VFPB_N)
	return
	end
			
			
			
			
			
	logical function MS_fast_pi_check(x, y)
      implicit NONE
c input
	real x, y
c local
	logical first /.true./
	integer j
c common for MS_FPB
	real MS_fp_boundary(5, 20)
	integer MS_FPB_N
	common /MS_FPB/ MS_FPB_N, MS_fp_boundary
c external
	logical area_check
c begin
	if(first) then
	   MS_FPB_N = 7
c	   MS_fp_boundary(1, 1) = 7.
c	   MS_fp_boundary(1, 1) = 5.5
	   MS_fp_boundary(1, 1) = 6.25
	   MS_fp_boundary(2, 1) = 0.5
	   MS_fp_boundary(1, 2) = 10.
	   MS_fp_boundary(2, 2) = 0.5
	   MS_fp_boundary(1, 3) = 12.57143
	   MS_fp_boundary(2, 3) = 0.62857
	   MS_fp_boundary(1, 4) = 11.18002
	   MS_fp_boundary(2, 4) = 2.01998
	   MS_fp_boundary(1, 5) = 10.
	   MS_fp_boundary(2, 5) = 1.9
	   MS_fp_boundary(1, 6) = 7.
	   MS_fp_boundary(2, 6) = 3.
c	   MS_fp_boundary(1, 7) = 4.91667
c	   MS_fp_boundary(2, 7) = 2.583335
c	   MS_fp_boundary(1, 7) = 3.66667
c	   MS_fp_boundary(2, 7) = 2.333335
	   MS_fp_boundary(1, 7) = 4.29167
	   MS_fp_boundary(2, 7) = 2.458335
	   Do j = 1, MS_FPB_N - 1
		MS_fp_boundary(3, j) = MS_fp_boundary(1, j+1) - MS_fp_boundary(1, j)
		MS_fp_boundary(4, j) = MS_fp_boundary(2, j+1) - MS_fp_boundary(2, j)
		MS_fp_boundary(5, j) = sqrt( MS_fp_boundary(3, j)**2 + MS_fp_boundary(4, j)**2 ) 
		MS_fp_boundary(3, j) = MS_fp_boundary(3, j) / MS_fp_boundary(5, j)
		MS_fp_boundary(4, j) = MS_fp_boundary(4, j) / MS_fp_boundary(5, j)
	   EndDo
	   MS_fp_boundary(3, MS_FPB_N) = MS_fp_boundary(1, 1) - MS_fp_boundary(1, MS_FPB_N)
	   MS_fp_boundary(4, MS_FPB_N) = MS_fp_boundary(2, 1) - MS_fp_boundary(2, MS_FPB_N)
	   MS_fp_boundary(5, MS_FPB_N) = sqrt( MS_fp_boundary(3, j)**2 + MS_fp_boundary(4, j)**2 ) 
	   MS_fp_boundary(3, MS_FPB_N) = MS_fp_boundary(3, MS_FPB_N) / MS_fp_boundary(5, MS_FPB_N)
	   MS_fp_boundary(4, MS_FPB_N) = MS_fp_boundary(4, MS_FPB_N) / MS_fp_boundary(5, MS_FPB_N)
	   first = .false.
	endif
	MS_fast_pi_check = area_check(x, y, MS_fp_boundary, MS_FPB_N)
	return
	end
			
			
			
			
			
	logical function MS_slow_pi_check(x, y)
      implicit NONE
c input
	real x, y
c external
	logical MS_slow_pi1_check, MS_slow_pi2_check
c begin
	MS_slow_pi_check = MS_slow_pi1_check(x, y)
	if(.not.MS_slow_pi_check) MS_slow_pi_check = MS_slow_pi2_check(x, y)
	return
	end
	
	
	
	
	
	logical function MS_slow_pi1_check(x, y)
      implicit NONE
c input
	real x, y
c local
	logical first /.true./
	integer j
c common for MS_SPB1
	real MS_sp1_boundary(5, 20)
	integer MS_SPB1_N
	common /MS_SPB1/ MS_SPB1_N, MS_sp1_boundary
c external
	logical area_check
c begin
	if(first) then
	   MS_SPB1_N = 7
	   MS_sp1_boundary(1, 1) = 12.57143
	   MS_sp1_boundary(2, 1) = 0.62857
	   MS_sp1_boundary(1, 2) = 20.
	   MS_sp1_boundary(2, 2) = 1.
	   MS_sp1_boundary(1, 3) = 60.
	   MS_sp1_boundary(2, 3) = 1.
	   MS_sp1_boundary(1, 4) = 60.
	   MS_sp1_boundary(2, 4) = 4.
	   MS_sp1_boundary(1, 5) = 40.
	   MS_sp1_boundary(2, 5) = 4.
	   MS_sp1_boundary(1, 6) = 19.39525
	   MS_sp1_boundary(2, 6) = 2.85529
	   MS_sp1_boundary(1, 7) = 11.18002
	   MS_sp1_boundary(2, 7) = 2.01998
	   Do j = 1, MS_SPB1_N - 1
		MS_sp1_boundary(3, j) = MS_sp1_boundary(1, j+1) - MS_sp1_boundary(1, j)
		MS_sp1_boundary(4, j) = MS_sp1_boundary(2, j+1) - MS_sp1_boundary(2, j)
		MS_sp1_boundary(5, j) = sqrt( MS_sp1_boundary(3, j)**2 + MS_sp1_boundary(4, j)**2 ) 
		MS_sp1_boundary(3, j) = MS_sp1_boundary(3, j) / MS_sp1_boundary(5, j)
		MS_sp1_boundary(4, j) = MS_sp1_boundary(4, j) / MS_sp1_boundary(5, j)
	   EndDo
	   MS_sp1_boundary(3, MS_SPB1_N) = MS_sp1_boundary(1, 1) - MS_sp1_boundary(1, MS_SPB1_N)
	   MS_sp1_boundary(4, MS_SPB1_N) = MS_sp1_boundary(2, 1) - MS_sp1_boundary(2, MS_SPB1_N)
	   MS_sp1_boundary(5, MS_SPB1_N) = sqrt( MS_sp1_boundary(3, j)**2 + MS_sp1_boundary(4, j)**2 ) 
	   MS_sp1_boundary(3, MS_SPB1_N) = MS_sp1_boundary(3, MS_SPB1_N) / MS_sp1_boundary(5, MS_SPB1_N)
	   MS_sp1_boundary(4, MS_SPB1_N) = MS_sp1_boundary(4, MS_SPB1_N) / MS_sp1_boundary(5, MS_SPB1_N)
	   first = .false.
	endif
	MS_slow_pi1_check = area_check(x, y, MS_sp1_boundary, MS_SPB1_N)
	return
	end
			
			
			
			
			
	logical function MS_slow_pi2_check(x, y)
      implicit NONE
c input
	real x, y
c local
	logical first /.true./
	integer j
c common for MS_SPB2
	real MS_sp2_boundary(5, 20)
	integer MS_SPB2_N
	common /MS_SPB2/ MS_SPB2_N, MS_sp2_boundary
c external
	logical area_check
c begin
	if(first) then
	   MS_SPB2_N = 4
	   MS_sp2_boundary(1, 1) = 2.
	   MS_sp2_boundary(2, 1) = 4.5
	   MS_sp2_boundary(1, 2) = 7.
	   MS_sp2_boundary(2, 2) = 3.
	   MS_sp2_boundary(1, 3) = 12.73919
	   MS_sp2_boundary(2, 3) = 4.38199
	   MS_sp2_boundary(1, 4) = 2.
	   MS_sp2_boundary(2, 4) = 7.
	   Do j = 1, MS_SPB2_N - 1
		MS_sp2_boundary(3, j) = MS_sp2_boundary(1, j+1) - MS_sp2_boundary(1, j)
		MS_sp2_boundary(4, j) = MS_sp2_boundary(2, j+1) - MS_sp2_boundary(2, j)
		MS_sp2_boundary(5, j) = sqrt( MS_sp2_boundary(3, j)**2 + MS_sp2_boundary(4, j)**2 ) 
		MS_sp2_boundary(3, j) = MS_sp2_boundary(3, j) / MS_sp2_boundary(5, j)
		MS_sp2_boundary(4, j) = MS_sp2_boundary(4, j) / MS_sp2_boundary(5, j)
	   EndDo
	   MS_sp2_boundary(3, MS_SPB2_N) = MS_sp2_boundary(1, 1) - MS_sp2_boundary(1, MS_SPB2_N)
	   MS_sp2_boundary(4, MS_SPB2_N) = MS_sp2_boundary(2, 1) - MS_sp2_boundary(2, MS_SPB2_N)
	   MS_sp2_boundary(5, MS_SPB2_N) = sqrt( MS_sp2_boundary(3, j)**2 + MS_sp2_boundary(4, j)**2 ) 
	   MS_sp2_boundary(3, MS_SPB2_N) = MS_sp2_boundary(3, MS_SPB2_N) / MS_sp2_boundary(5, MS_SPB2_N)
	   MS_sp2_boundary(4, MS_SPB2_N) = MS_sp2_boundary(4, MS_SPB2_N) / MS_sp2_boundary(5, MS_SPB2_N)
	   first = .false.
	endif
	MS_slow_pi2_check = area_check(x, y, MS_sp2_boundary, MS_SPB2_N)
	return
	end
			
			
			
			
			
	logical function MS_proton_check(x, y)
      implicit NONE
c input
	real x, y
c local
	logical first /.true./
	integer j
c common for MS_PB
	real MS_p_boundary(5, 20)
	integer MS_PB_N
	common /MS_PB/ MS_PB_N, MS_p_boundary
c external
	logical area_check
c begin
	if(first) then
	   MS_PB_N = 8
	   MS_p_boundary(1, 1) = 2.
	   MS_p_boundary(2, 1) = 10.
	   MS_p_boundary(1, 2) = 10.
	   MS_p_boundary(2, 2) = 7.5
	   MS_p_boundary(1, 3) = 17.69684
	   MS_p_boundary(2, 3) = 5.57579
	   MS_p_boundary(1, 4) = 40.
	   MS_p_boundary(2, 4) = 4.
	   MS_p_boundary(1, 5) = 45.
	   MS_p_boundary(2, 5) = 4.
	   MS_p_boundary(1, 6) = 45.
	   MS_p_boundary(2, 6) = 5.918605
	   MS_p_boundary(1, 7) = 17.
	   MS_p_boundary(2, 7) = 9.5
	   MS_p_boundary(1, 8) = 2.
	   MS_p_boundary(2, 8) = 14.
	   Do j = 1, MS_PB_N - 1
		MS_p_boundary(3, j) = MS_p_boundary(1, j+1) - MS_p_boundary(1, j)
		MS_p_boundary(4, j) = MS_p_boundary(2, j+1) - MS_p_boundary(2, j)
		MS_p_boundary(5, j) = sqrt( MS_p_boundary(3, j)**2 + MS_p_boundary(4, j)**2 ) 
		MS_p_boundary(3, j) = MS_p_boundary(3, j) / MS_p_boundary(5, j)
		MS_p_boundary(4, j) = MS_p_boundary(4, j) / MS_p_boundary(5, j)
	   EndDo
	   MS_p_boundary(3, MS_PB_N) = MS_p_boundary(1, 1) - MS_p_boundary(1, MS_PB_N)
	   MS_p_boundary(4, MS_PB_N) = MS_p_boundary(2, 1) - MS_p_boundary(2, MS_PB_N)
	   MS_p_boundary(5, MS_PB_N) = sqrt( MS_p_boundary(3, j)**2 + MS_p_boundary(4, j)**2 ) 
	   MS_p_boundary(3, MS_PB_N) = MS_p_boundary(3, MS_PB_N) / MS_p_boundary(5, MS_PB_N)
	   MS_p_boundary(4, MS_PB_N) = MS_p_boundary(4, MS_PB_N) / MS_p_boundary(5, MS_PB_N)
	   first = .false.
	endif
	MS_proton_check = area_check(x, y, MS_p_boundary, MS_PB_N)
	return
	end
			
			
			
			
			
	logical function fast_pi_check(x, y)
      implicit NONE
c input
	real x, y
c local
	logical first /.true./
	integer j
c common for FPB
	real fp_boundary(5, 20)
	integer FPB_N
	common /FPB/ FPB_N, fp_boundary
c external
	logical area_check
c begin
	if(first) then
	   FPB_N = 5
	   fp_boundary(1, 1) = 2.
	   fp_boundary(2, 1) = 0.5
	   fp_boundary(1, 2) = 10.
	   fp_boundary(2, 2) = 0.5
	   fp_boundary(1, 3) = 20.
	   fp_boundary(2, 3) = 1.
	   fp_boundary(1, 4) = 7.
	   fp_boundary(2, 4) = 3.
	   fp_boundary(1, 5) = 2.
	   fp_boundary(2, 5) = 2.
	   Do j = 1, FPB_N - 1
		fp_boundary(3, j) = fp_boundary(1, j+1) - fp_boundary(1, j)
		fp_boundary(4, j) = fp_boundary(2, j+1) - fp_boundary(2, j)
		fp_boundary(5, j) = sqrt( fp_boundary(3, j)**2 + fp_boundary(4, j)**2 ) 
		fp_boundary(3, j) = fp_boundary(3, j) / fp_boundary(5, j)
		fp_boundary(4, j) = fp_boundary(4, j) / fp_boundary(5, j)
	   EndDo
	   fp_boundary(3, FPB_N) = fp_boundary(1, 1) - fp_boundary(1, FPB_N)
	   fp_boundary(4, FPB_N) = fp_boundary(2, 1) - fp_boundary(2, FPB_N)
	   fp_boundary(5, FPB_N) = sqrt( fp_boundary(3, j)**2 + fp_boundary(4, j)**2 ) 
	   fp_boundary(3, FPB_N) = fp_boundary(3, FPB_N) / fp_boundary(5, FPB_N)
	   fp_boundary(4, FPB_N) = fp_boundary(4, FPB_N) / fp_boundary(5, FPB_N)
	   first = .false.
	endif
	fast_pi_check = area_check(x, y, fp_boundary, FPB_N)
	return
	end
			
			
			
			
			
	logical function slow_pi_check(x, y)
      implicit NONE
c input
	real x, y
c local
	logical first /.true./
	integer j
c common for SPB
	real sp_boundary(5, 20)
	integer SPB_N
	common /SPB/ SPB_N, sp_boundary
c external
	logical area_check
c begin
	if(first) then
	   SPB_N = 9
	   sp_boundary(1, 1) = 2.
	   sp_boundary(2, 1) = 4.5
	   sp_boundary(1, 2) = 7.
	   sp_boundary(2, 2) = 3.
	   sp_boundary(1, 3) = 20.
	   sp_boundary(2, 3) = 1.
	   sp_boundary(1, 4) = 60.
	   sp_boundary(2, 4) = 1.
	   sp_boundary(1, 5) = 60.
	   sp_boundary(2, 5) = 4.
	   sp_boundary(1, 6) = 40.
	   sp_boundary(2, 6) = 4.
	   sp_boundary(1, 7) = 13.
	   sp_boundary(2, 7) = 2.5
	   sp_boundary(1, 8) = 16.02602
	   sp_boundary(2, 8) = 3.5807215
	   sp_boundary(1, 9) = 2.
	   sp_boundary(2, 9) = 7.
	   Do j = 1, SPB_N - 1
		sp_boundary(3, j) = sp_boundary(1, j+1) - sp_boundary(1, j)
		sp_boundary(4, j) = sp_boundary(2, j+1) - sp_boundary(2, j)
		sp_boundary(5, j) = sqrt( sp_boundary(3, j)**2 + sp_boundary(4, j)**2 ) 
		sp_boundary(3, j) = sp_boundary(3, j) / sp_boundary(5, j)
		sp_boundary(4, j) = sp_boundary(4, j) / sp_boundary(5, j)
	   EndDo
	   sp_boundary(3, SPB_N) = sp_boundary(1, 1) - sp_boundary(1, SPB_N)
	   sp_boundary(4, SPB_N) = sp_boundary(2, 1) - sp_boundary(2, SPB_N)
	   sp_boundary(5, SPB_N) = sqrt( sp_boundary(3, j)**2 + sp_boundary(4, j)**2 ) 
	   sp_boundary(3, SPB_N) = sp_boundary(3, SPB_N) / sp_boundary(5, SPB_N)
	   sp_boundary(4, SPB_N) = sp_boundary(4, SPB_N) / sp_boundary(5, SPB_N)
	   first = .false.
	endif
	slow_pi_check = area_check(x, y, sp_boundary, SPB_N)
	return
	end
			
			
			
			
			
	logical function proton_check(x, y)
      implicit NONE
c input
	real x, y
c local
	logical first /.true./
	integer j
c common for PB
	real p_boundary(5, 20)
	integer PB_N
	common /PB/ PB_N, p_boundary
c external
	logical area_check
c begin
	if(first) then
	   PB_N = 9
	   p_boundary(1, 1) = 2.
	   p_boundary(2, 1) = 10.
	   p_boundary(1, 2) = 10.
	   p_boundary(2, 2) = 7.5
	   p_boundary(1, 3) = 20.
	   p_boundary(2, 3) = 5.
	   p_boundary(1, 4) = 13.
	   p_boundary(2, 4) = 2.5
	   p_boundary(1, 5) = 40.
	   p_boundary(2, 5) = 4.
	   p_boundary(1, 6) = 45.
	   p_boundary(2, 6) = 4.
	   p_boundary(1, 7) = 45.
	   p_boundary(2, 7) = 5.918605
	   p_boundary(1, 8) = 17.
	   p_boundary(2, 8) = 9.5
	   p_boundary(1, 9) = 2.
	   p_boundary(2, 9) = 14.
	   Do j = 1, PB_N - 1
		p_boundary(3, j) = p_boundary(1, j+1) - p_boundary(1, j)
		p_boundary(4, j) = p_boundary(2, j+1) - p_boundary(2, j)
		p_boundary(5, j) = sqrt( p_boundary(3, j)**2 + p_boundary(4, j)**2 ) 
		p_boundary(3, j) = p_boundary(3, j) / p_boundary(5, j)
		p_boundary(4, j) = p_boundary(4, j) / p_boundary(5, j)
	   EndDo
	   p_boundary(3, PB_N) = p_boundary(1, 1) - p_boundary(1, PB_N)
	   p_boundary(4, PB_N) = p_boundary(2, 1) - p_boundary(2, PB_N)
	   p_boundary(5, PB_N) = sqrt( p_boundary(3, j)**2 + p_boundary(4, j)**2 ) 
	   p_boundary(3, PB_N) = p_boundary(3, PB_N) / p_boundary(5, PB_N)
	   p_boundary(4, PB_N) = p_boundary(4, PB_N) / p_boundary(5, PB_N)
	   first = .false.
	endif
	proton_check = area_check(x, y, p_boundary, PB_N)
	return
	end
			
			
			
			
			
	logical function deuteron_check(x, y)
      implicit NONE
c input
	real x, y
c local
	logical first /.true./
	integer j
c common for DB
	real d_boundary(5, 20)
	integer DB_N
	common /DB/ DB_N, d_boundary
c external
	logical area_check
c begin
	if(first) then
  	   DB_N = 8
	   d_boundary(1, 1) = 2.
	   d_boundary(2, 1) = 14.
	   d_boundary(1, 2) = 17.
	   d_boundary(2, 2) = 9.5
	   d_boundary(1, 3) = 45.
	   d_boundary(2, 3) = 5.918605
	   d_boundary(1, 4) = 55.
	   d_boundary(2, 4) = 5.918605
	   d_boundary(1, 5) = 55.
	   d_boundary(2, 5) = 9.75
	   d_boundary(1, 6) = 45.
	   d_boundary(2, 6) = 9.75
	   d_boundary(1, 7) = 22.11516
	   d_boundary(2, 7) = 12.419898
	   d_boundary(1, 8) = 2.
	   d_boundary(2, 8) = 17.936667
	   Do j = 1, DB_N - 1
		d_boundary(3, j) = d_boundary(1, j+1) - d_boundary(1, j)
		d_boundary(4, j) = d_boundary(2, j+1) - d_boundary(2, j)
		d_boundary(5, j) = sqrt( d_boundary(3, j)**2 + d_boundary(4, j)**2 ) 
		d_boundary(3, j) = d_boundary(3, j) / d_boundary(5, j)
		d_boundary(4, j) = d_boundary(4, j) / d_boundary(5, j)
	   EndDo
	   d_boundary(3, DB_N) = d_boundary(1, 1) - d_boundary(1, DB_N)
	   d_boundary(4, DB_N) = d_boundary(2, 1) - d_boundary(2, DB_N)
	   d_boundary(5, DB_N) = sqrt( d_boundary(3, j)**2 + d_boundary(4, j)**2 ) 
	   d_boundary(3, DB_N) = d_boundary(3, DB_N) / d_boundary(5, DB_N)
	   d_boundary(4, DB_N) = d_boundary(4, DB_N) / d_boundary(5, DB_N)
	   first = .false.
	endif
	deuteron_check = area_check(x, y, d_boundary, DB_N)
	return
	end
			
			
			
			
			
 	logical function area_check(x, y, boundary, N)
      implicit NONE
c input
	real x, y, boundary(5, 20)
	integer N
c local
	integer count, j
	real vr, wv, s, t
	real r(2), v(2), ar(2), vwv(2), vww(2)
	real x0 /0./, y0 /0./
c begin
	r(1) = x0
	r(2) = y0
	v(1) = x - x0
	v(2) = y - y0
	vr = sqrt( v(1)**2 + v(2)**2 )
	v(1) = v(1) / vr
	v(2) = v(2) / vr
	count = 0
c	write(*,*) '### r, v ', r(1), r(2), v(1), v(2)
	Do j = 1, N
	   ar(1) = boundary(1, j) - r(1)
	   ar(2) = boundary(2, j) - r(2) 
	   wv = v(1) * boundary(3, j) + v(2) * boundary(4, j)
	   if( 1. - wv**2 .ne. 0.) then
	      vwv(1) = v(1) - wv * boundary(3, j) 
	      vwv(2) = v(2) - wv * boundary(4, j)
	      t =  ( ar(1) * vwv(1) + ar(2) * vwv(2) ) / ( 1. - wv**2 )
	      vww(1) = boundary(3, j) - wv * v(1)
	      vww(2) = boundary(4, j) - wv * v(2)
	      s =  - ( ar(1) * vww(1) + ar(2) * vww(2) ) / ( 1. - wv**2 )
c	write(*,*) 'j:  a, w ', j, ' :', boundary(1, j), boundary(2, j), boundary(3, j), boundary(4, j)
c	write(*,*) '    0<t<vr, 0<s< wa      - ', '0<',t,' <', vr, ':   0<',s,' <', boundary(5, j)
	      if( s.lt.boundary(5, j) .and. t.lt.vr
     &	   .and. t.ge.0. .and. s.gt.0. ) then
c	write(*,*) ' hit!!'
     		   count = count + 1
     		endif
     	   endif
	enddo
c	write(*,*) 'count = ', count
	count = count - int(count/2)*2
	if(count .eq. 1) then
	   area_check = .true.
c	write(*,*) '### This point is inside the boundary!'
	else
	   area_check = .false.
	endif
	return
	end
		
		
		
      subroutine event_analysis
      implicit NONE
c input
c ... common /PIDs/
      integer PID_pion, PID_pi0, PID_proton, PID_neutron,
     &	PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
      common /PIDs/ PID_pion, PID_pi0, PID_proton, PID_neutron, 
     &      PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
c ... common /kaon/ for kaon trajectory
      real  kaon_point(3) ! position of the K-reaction
      real  kt_vector(3) ! trajectory of the kaon injection
      real  kt_point(3) ! position of the K-reaction
      real  K_stop_time, K_flight_time, dL_T0
      common /kaon/ kaon_point, kt_point, kt_vector, K_stop_time, K_flight_time, dL_T0
c ... common for target position
	common /trg/ target_center, tm, tta, tc
	real target_center, tm, tta, tc
c ... common /RIDS/ reaction ID
	integer KN, tnL, tnS, pi_abs, SLC, QFL, QFS, pi_mono, CBG
	common /RIDS/ KN, tnL, tnS, pi_abs, SLC, QFL, QFS, pi_mono, CBG
c ... common for setup
	integer n_start, n_end
	common /ncount/ n_start, n_end
	common /vetoc/ VDC_VETO_start, VDC_VETO_end, BDC_VETO_start, BDC_VETO_end
	integer VDC_VETO_start, VDC_VETO_end, BDC_VETO_start, BDC_VETO_end
c ... common /setup/ for simulation run-condition, counter resolution and its efficiency
	integer FF_knucl_decay, multi_set, k_decay_set
	logical Kstop_cor
      real  delta_x, delta_y, delta_T0, delta_NC, TC_eff, T0_eff
      common /setup/ delta_x, delta_y, delta_T0, delta_NC, TC_eff, T0_eff,
     &	FF_knucl_decay, multi_set, k_decay_set, Kstop_cor
c ... common /YPV/ hyperon decay point and vector
	real Y_point(3), Y_v(3), v_dist_L, v_dist_S
	integer Y_ID
      common /YPV/ Y_ID, Y_point, Y_v, v_dist_L, v_dist_S
c ...	common /NC_data/ for neutron counter buffer 
	real BS(20, 2), BSv(20, 3), BSp(20, 3)
	integer BSptr, iBS(20)
	logical LBS(20, 2)
	common /NC_data/ BSptr, iBS, LBS(20, 2), BS, BSv, BSp
c ... common /TC_data/ for TC info.
	integer PID_trg
	real dL_thin, dL_thick, p_trg, t_trg, v_trg(3), pt_trg(3) 
	common /TC_data/ PID_trg, dL_thin, dL_thick, p_trg, t_trg, v_trg, pt_trg
c common for debug
	logical debug_F
      integer ev_NUM, ev_error
      common /DeBugging/ debug_F, ev_NUM, ev_error
c>>c common for debug2
c>>	logical knucl_f
c>>	integer kdecay
c>>      common /DeBugging2/ knucl_f, kdecay
c local
	logical proton_PID, fast_pi_PID, slow_pi_PID, deuteron_PID 
	logical Dbl_Arm_Hit, Chrg_Hit
	integer n_reactionID
	integer j, iv
	real v_CA_v_n, v_Y_v_n, v_trg_v_n, b_trg
	real norm_dL_thin, norm_dL_thick
      real diffv(3), dca_v(3), zk, zkt
      real vertex_diff, time_n, DCA
	real dcross(3), Kp_cross(3), kv_cross(3)
	real tof_tof, T_n_tof, B_KN_tof, p_n_tof, v_n_tof(3), beta_inv_tof, NFL_tof
      real p_n, v_n(3), point_n(3), t_trg_tof, b_trg_tof
      logical MS_vfast_pi_PID, MS_fast_pi_PID, MS_slow_pi_PID, MS_proton_PID
c ... external
	real n_efficiency, rn, dT_cor, gauss
c begin
	DCA = vertex_diff(kt_point, kt_vector, pt_trg, v_trg, 
     &			dcross, diffv) * 10.
	b_trg = t_trg / ( sqrt( (pt_trg(1)-dcross(1))**2 
     &		+ (pt_trg(2)-dcross(2))**2  
     &		+ (pt_trg(3)-dcross(3))**2 ) / 29.9792458 )
	t_trg_tof = t_trg + gauss(0., delta_T0) + gauss(0., delta_NC)
	if(Kstop_cor) t_trg_tof = t_trg_tof + 
     &	K_flight_time - dT_cor(kp_cross, tc)
	b_trg_tof = t_trg_tof / ( sqrt( (pt_trg(1)-dcross(1))**2 
     &		+ (pt_trg(2)-dcross(2))**2  
     &		+ (pt_trg(3)-dcross(3))**2 ) / 29.9792458 )
	Do iv = 1, 3
	   kp_cross(iv) = dcross(iv) 		! unit cm
	   kv_cross(iv) = diffv(iv) * 10.	! unit mm
	   dca_v(iv) = kv_cross(iv) / DCA
	endDo

c fill spectra for trigger
c	write(*,*) 'fill spectra for trigger'
c vertex-z by track
	zkt = (kaon_point(3) - tc) * 10.
c vertex-z by track
	zk = (kp_cross(3) - tc) * 10.
c ... kaon
	call td_hf1(1, zkt, 1.)
	call td_hf2(2, zkt, K_flight_time, 1.)
	call td_hf2(3, zkt, dL_T0, 1.)
	call td_hf1(4, zk, 1.)
	call td_hf2(5, zk, K_flight_time, 1.)
	call td_hf2(6, zk, K_flight_time - dT_cor(kp_cross, tc), 1.)
	call td_hf2(7, zkt, DCA, 1.)
c ... trigger particle
	norm_dL_thin = dL_thin*abs(v_trg(2))
	norm_dL_thick = dL_thick*abs(v_trg(2))
	call PIDdef(b_trg, norm_dL_thick, norm_dL_thin, 
     &	fast_pi_PID, slow_pi_PID, deuteron_PID, proton_PID,
     &	MS_vfast_pi_PID, MS_fast_pi_PID, MS_slow_pi_PID, MS_proton_PID)
	if(PID_trg .eq. PID_pion) then
	   call td_hf1( 104, v_trg(1), 1.)
	   call td_hf1( 105, v_trg(2), 1.)
	   call td_hf1( 106, v_trg(3), 1.)
	   call td_hf2( 113, pt_trg(3), pt_trg(1), 1.)
	   call td_hf1( 116, pt_trg(3), 1.)
	else if(PID_trg .eq. -PID_pion) then
	   call td_hf1( 107, v_trg(1), 1.)
	   call td_hf1( 108, v_trg(2), 1.)
	   call td_hf1( 109, v_trg(3), 1.)
	   call td_hf2( 114, pt_trg(3), pt_trg(1), 1.)
	   call td_hf1( 117, pt_trg(3), 1.)
	else if(abs(PID_trg) .eq. PID_deuteron) then
	   call td_hf1( 110, v_trg(1), 1.)
	   call td_hf1( 111, v_trg(2), 1.)
	   call td_hf1( 112, v_trg(3), 1.)
	   call td_hf2( 115, pt_trg(3), pt_trg(1), 1.)
	   call td_hf1( 118, pt_trg(3), 1.)
	endif

c ... vertex
      call td_hf1(122, kv_cross(1), 1.)
      call td_hf1(123, kv_cross(2), 1.)
      call td_hf1(124, kv_cross(3), 1.)
	call td_hf2(125, kp_cross(1), kp_cross(2), 1.)
	call td_hf2(126, kp_cross(3)-tc, kp_cross(1), 1.)
	call td_hf2(127, kp_cross(3)-tc, kp_cross(2), 1.)
c target fidutial cut
	if ( abs(kp_cross(3) - tc) .gt. 16./2. ) return
	if ( sqrt(kp_cross(1)**2 + kp_cross(2)**2) .gt. 13. ) return 

c check for each neutron
      do j = 1, BSptr
c ...    restore neutron info
	   time_n = BS(BSptr, 1)
	   p_n = BS(BSptr, 2) 
	   point_n(1) = BSp(BSptr, 1) 
	   point_n(2) = BSp(BSptr, 2) 
	   point_n(3) = BSp(BSptr, 3) 
	   v_n(1) = BSv(BSptr, 1) 
	   v_n(2) = BSv(BSptr, 2)
	   v_n(3) = BSv(BSptr, 3)
	   n_reactionID = iBS(BSptr) 
	   Dbl_Arm_Hit = LBS(BSptr, 1) 
	   Chrg_Hit = LBS(BSptr, 2) 
c>>			if( knucl_f ) then
c>>			   if( n_reactionID .eq. -1) kdecay = kdecay + 1
c>>			else 
c>>			   if(kdecay.ne.0) write(*,*) 'ev_NUM, kdecay = ', ev_NUM, kdecay
c>>			   if( n_reactionID .eq. -1) then
c>>			   	write(*,*) 'What ???', ev_NUM
c>>			   	stop
c>>			   endif
c>>			   kdecay = 0
c>>			endif
c 
	   call neutron_kin( kp_cross, point_n, time_n, tc, 
     &	tof_tof, NFL_tof, beta_inv_tof, T_n_tof, p_n_tof, B_KN_tof, v_n_tof)

	   if(dL_thick .gt. 1.) then
	     call td_hf2(100, norm_dL_thick, norm_dL_thin, 1.)
	     if(PID_trg .eq. PID_proton) call td_hf2(101, norm_dL_thick, norm_dL_thin, 1.)
c	     if(PID_trg .eq. PID_deuteron) call td_hf2(160, norm_dL_thick, norm_dL_thin, 1.)
	     if(abs(PID_trg) .eq. PID_pion) call td_hf2(160, norm_dL_thick, norm_dL_thin, 1.)
c>	     if(proton_PID) call td_hf2(101, norm_dL_thick, norm_dL_thin, 1.)
c>	     if(deuteron_PID) call td_hf2(160, norm_dL_thick, norm_dL_thin, 1.)
	     if(slow_pi_PID) call td_hf2(161, norm_dL_thick, norm_dL_thin, 1.)
	     if(MS_vfast_pi_PID) call td_hf2(163, norm_dL_thick, norm_dL_thin, 1.)
	     if(MS_fast_pi_PID) call td_hf2(164, norm_dL_thick, norm_dL_thin, 1.)
	     if(MS_slow_pi_PID) call td_hf2(165, norm_dL_thick, norm_dL_thin, 1.)
	     if(MS_proton_PID) call td_hf2(166, norm_dL_thick, norm_dL_thin, 1.)
c	     if(abs(PID_trg) .eq. PID_pion) then
	     if(fast_pi_PID) then
	       call td_hf2(102, norm_dL_thick, norm_dL_thin, 1.)
	       call td_hf2(130, p_trg, norm_dL_thin, 1.)
	       call td_hf2(131, p_trg, norm_dL_thick, 1.)
	       if(norm_dL_thin.gt.1.2 .and. norm_dL_thin.lt.1.7) call td_hf2(162, p_trg, norm_dL_thick, 1.)
	       call td_hf2(120, v_dist_L, DCA, 1.)
	       call td_hf2(121, v_dist_S, DCA, 1.)
	     endif
	   else
	   endif
	     call td_hf2(800, b_trg_tof, norm_dL_thin, 1.)
	     if(PID_trg .eq. PID_proton) call td_hf2(801, b_trg_tof, norm_dL_thin, 1.)
	     if(MS_fast_pi_PID) call td_hf2(802, b_trg_tof, norm_dL_thin, 1.)
	     if(PID_trg .eq. PID_deuteron) call td_hf2(803, b_trg_tof, norm_dL_thin, 1.)
	   call td_hf1_RID( 900, tof_tof, n_reactionID )
	   call td_hf1_RID( 920, beta_inv_tof, n_reactionID )
	   
c ...    define how to treat NC veto
c	   if(Dbl_Arm_Hit) then
c	      go to 10
c	   endif
	   if (Chrg_Hit) go to 10

	   if(p_n_tof .lt. 120.) go to 10

	   if(b_trg_tof .gt. 2.2 .or. b_trg_tof .lt. 0.6) MS_fast_pi_PID = .false.

	   if (p_n .gt. 100.) then
	      if(PID_trg .eq. PID_pion) then
	   	   call td_hf1_RID( 1220, p_trg, n_reactionID )
	      else if(PID_trg .eq. -PID_pion) then
	   	   call td_hf1_RID( 1240, p_trg, n_reactionID )
	      else if((PID_trg .eq. PID_proton) .or. (PID_trg .eq. PID_deuteron)) then
	   	   call td_hf1_RID( 1260, p_trg, n_reactionID )
		endif
	   endif

c ...	   fill spectra for each neutron 
         call td_hf2(1200, p_n, p_n_tof, 1.)
	   call td_hf1_RID( 140, DCA, n_reactionID ) 
	   if(n_reactionID .eq. KN) call td_hf2(119, norm_dL_thick, norm_dL_thin, 1.)

c ...	   anglular correlations
c	... ^v_trg * ^v_n_tof (cosQ obetween trigger and neutron)
	   v_trg_v_n = v_trg(1)*v_n_tof(1) +  v_trg(2)*v_n_tof(2) +  v_trg(3)*v_n_tof(3) 
c	... v_CA * ^v_n_tof
	   v_CA_v_n = (dca_v(1)*v_n_tof(1) + dca_v(2)*v_n_tof(2) + dca_v(3)*v_n_tof(3))
     &	* DCA
c	... ^v_Y * ^v_n
	   v_Y_v_n = Y_v(1)*v_n(1) + Y_v(2)*v_n(2) + Y_v(3)*v_n(3)
c 	... fill corr
	   call td_hf2(2000, v_CA_v_n, v_Y_v_n, 1.)
	   if( DCA .gt. 10. ) call td_hf2(2001, v_CA_v_n, v_Y_v_n, 1.)
	   if((proton_PID) .or. (deuteron_PID)) then 
		call td_hf1_RID( 2060, v_trg_v_n, n_reactionID )
		if(p_n_tof.gt.400.) call td_hf1_RID( 2080, v_trg_v_n, n_reactionID )
	      if(n_reactionID .eq. KN) then
		   if(proton_PID) then 
		      call td_hf2(2002, p_n_tof, v_trg_v_n, 1.)
		      call td_hf2(2006, p_trg, v_trg_v_n, 1.)
		      call td_hf2(2004, norm_dL_thin, v_trg_v_n, 1.)
	         else
		      call td_hf2(2102, p_n_tof, v_trg_v_n, 1.)
		      call td_hf2(2106, p_trg, v_trg_v_n, 1.)
		      call td_hf2(2104, norm_dL_thin, v_trg_v_n, 1.)
	         endif
	      else
		   if(proton_PID) then 
		      call td_hf2(2003, p_n_tof, v_trg_v_n, 1.)
		      call td_hf2(2005, norm_dL_thin, v_trg_v_n, 1.)
	         else
		      call td_hf2(2103, p_n_tof, v_trg_v_n, 1.)
		      call td_hf2(2105, norm_dL_thin, v_trg_v_n, 1.)
	         endif
	      endif
	   endif
	   if(abs(PID_trg) .eq. PID_pion) call td_hf1_RID( 2020, v_CA_v_n, n_reactionID )
	   if(MS_fast_pi_PID) call td_hf1_RID( 2040, v_CA_v_n, n_reactionID )
	   if(MS_slow_pi_PID) call td_hf1_RID( 2120, v_CA_v_n, n_reactionID )
	   call td_hf1_RID( 2140, v_CA_v_n, n_reactionID )

c    ... neutron with pion trigger without n_efficiency 
	   if(slow_pi_PID.or.fast_pi_PID) then 
		call td_hf1_RID( 3020, p_n_tof, n_reactionID )
	      if(slow_pi_PID) then 
		   call td_hf1(3004, norm_dL_thin, 1.)
		   call td_hf1_RID( 3040, p_n_tof, n_reactionID )
		   if(v_CA_v_n .ge. -60. .and. v_CA_v_n .lt. -10. ) then
		      call td_hf1_RID( 3080, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. -10. .and. v_CA_v_n .lt. -5. ) then
		      call td_hf1_RID( 3100, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. -5. .and. v_CA_v_n .lt. 5. ) then
		      call td_hf1_RID( 3120, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. 5. .and. v_CA_v_n .lt. 10. ) then
		      call td_hf1_RID( 3140, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. 10. .and. v_CA_v_n .lt. 60. ) then
		      call td_hf1_RID( 3160, p_n_tof, n_reactionID )
	         endif
	      else if(fast_pi_PID) then 
		   call td_hf1(3006, norm_dL_thin, 1.)
		   call td_hf1_RID( 3060, p_n_tof, n_reactionID )
		   if(v_CA_v_n .ge. -60. .and. v_CA_v_n .lt. -10. ) then
		      call td_hf1_RID( 3180, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. -10. .and. v_CA_v_n .lt. -5. ) then
		      call td_hf1_RID( 3200, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. -5. .and. v_CA_v_n .lt. 5. ) then
		      call td_hf1_RID( 3220, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. 5. .and. v_CA_v_n .lt. 10. ) then
		      call td_hf1_RID( 3240, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. 10. .and. v_CA_v_n .lt. 60. ) then
		      call td_hf1_RID( 3260, p_n_tof, n_reactionID )
	         endif
	      endif
	   endif

c ... neutron with proton trigger without n_efficiency
	   if((proton_PID) .or. (deuteron_PID)) then 
		if(norm_dL_thin .lt. 4.) then
		   call td_hf1( 3301, p_trg, 1.)
		else
		   call td_hf1( 3302, p_trg, 1.)
		endif
		call td_hf1_RID( 3320, p_n_tof, n_reactionID )
	      if(v_trg_v_n .lt. 0.) then 
		   call td_hf1_RID( 3340, p_n_tof, n_reactionID )
		   if(norm_dL_thin .lt. 4.) then
		      call td_hf1_RID( 3380, p_n_tof, n_reactionID )
		   else
		      call td_hf1_RID( 3400, p_n_tof, n_reactionID )
		   endif
	      else
		   call td_hf1_RID( 3360, p_n_tof, n_reactionID )
		   if(norm_dL_thin .lt. 4.) then
		      call td_hf1_RID( 3420, p_n_tof, n_reactionID )
		   else
		      call td_hf1_RID( 3440, p_n_tof, n_reactionID )
		   endif
	      endif
	   endif

	   if( rn() .gt. n_efficiency(p_n) ) go to 10

c    ... neutron with pion trigger with n_efficiency
	   if(slow_pi_PID.or.fast_pi_PID) then 
		call td_hf1_RID( 4020, p_n_tof, n_reactionID )
	      if(p_trg .le. 100.) then 
		   call td_hf1_RID( 4040, p_n_tof, n_reactionID )
		   if(v_CA_v_n .ge. -60. .and. v_CA_v_n .lt. -10. ) then
		      call td_hf1_RID( 4080, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. -10. .and. v_CA_v_n .lt. -5. ) then
		      call td_hf1_RID( 4100, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. -5. .and. v_CA_v_n .lt. 5. ) then
		      call td_hf1_RID( 4120, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. 5. .and. v_CA_v_n .lt. 10. ) then
		      call td_hf1_RID( 4140, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. 10. .and. v_CA_v_n .lt. 60. ) then
		      call td_hf1_RID( 4160, p_n_tof, n_reactionID )
	         endif
	      else if(p_trg .ge. 135. .and. p_trg .le. 250.) then 
		   call td_hf1_RID( 4060, p_n_tof, n_reactionID )
		   if(v_CA_v_n .ge. -60. .and. v_CA_v_n .lt. -10. ) then
		      call td_hf1_RID( 4180, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. -10. .and. v_CA_v_n .lt. -5. ) then
		      call td_hf1_RID( 4200, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. -5. .and. v_CA_v_n .lt. 5. ) then
		      call td_hf1_RID( 4220, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. 5. .and. v_CA_v_n .lt. 10. ) then
		      call td_hf1_RID( 4240, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. 10. .and. v_CA_v_n .lt. 60. ) then
		      call td_hf1_RID( 4260, p_n_tof, n_reactionID )
	         endif
	      endif
	   endif

c    ... neutron with pion trigger with n_efficiency
	   if(slow_pi_PID.or.fast_pi_PID) then 
		if(slow_pi_PID) then
		   call td_hf1_RID( 5040, p_n_tof, n_reactionID )
		   if(v_CA_v_n .ge. -60. .and. v_CA_v_n .lt. -5. ) then
		      call td_hf1_RID( 5080, p_n_tof, n_reactionID )
		      if(v_CA_v_n .ge. -10. .and. v_CA_v_n .lt. -5. ) then
		         call td_hf1_RID( 5100, p_n_tof, n_reactionID )
		      endif
		   else if(v_CA_v_n .ge. -5. .and. v_CA_v_n .lt. 5. ) then
		      call td_hf1_RID( 5120, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. 5. .and. v_CA_v_n .lt. 60. ) then
		      call td_hf1_RID( 5160, p_n_tof, n_reactionID )
		      if(v_CA_v_n .ge. 5. .and. v_CA_v_n .lt. 10. ) then
		         call td_hf1_RID( 5140, p_n_tof, n_reactionID )
	            endif
	         endif
	      else 
		   call td_hf1_RID( 5060, p_n_tof, n_reactionID )
		   if(v_CA_v_n .ge. -60. .and. v_CA_v_n .lt. -5. ) then
		      call td_hf1_RID( 5180, p_n_tof, n_reactionID )
		      if(v_CA_v_n .ge. -10. .and. v_CA_v_n .lt. -5. ) then
		         call td_hf1_RID( 5200, p_n_tof, n_reactionID )
		      endif
		   else if(v_CA_v_n .ge. -5. .and. v_CA_v_n .lt. 5. ) then
		      call td_hf1_RID( 5220, p_n_tof, n_reactionID )
		   else if(v_CA_v_n .ge. 5. .and. v_CA_v_n .lt. 60. ) then
		      call td_hf1_RID( 5260, p_n_tof, n_reactionID )
		      if(v_CA_v_n .ge. 10. .and. v_CA_v_n .lt. 60. ) then
		         call td_hf1_RID( 5240, p_n_tof, n_reactionID )
	            endif
	         endif
	      endif
	   endif

	   if(MS_vfast_pi_PID) then
		if(v_CA_v_n .ge. -60. .and. v_CA_v_n .lt. -5. ) then
		         call td_hf1_RID( 6000, p_n_tof, n_reactionID )
		else if(v_CA_v_n .ge. -5. .and. v_CA_v_n .lt. 5. ) then
		         call td_hf1_RID( 6020, p_n_tof, n_reactionID )
		else if(v_CA_v_n .ge. 5. .and. v_CA_v_n .lt. 60. ) then
		         call td_hf1_RID(6040, p_n_tof, n_reactionID )
		endif
	   else if(MS_fast_pi_PID) then
		if(v_CA_v_n .ge. -60. .and. v_CA_v_n .lt. -5. ) then
		         call td_hf1_RID( 6060, p_n_tof, n_reactionID )
		else if(v_CA_v_n .ge. -5. .and. v_CA_v_n .lt. 5. ) then
		         call td_hf1_RID( 6080, p_n_tof, n_reactionID )
		else if(v_CA_v_n .ge. 5. .and. v_CA_v_n .lt. 60. ) then
		         call td_hf1_RID(6100, p_n_tof, n_reactionID )
		endif
	   else if(MS_slow_pi_PID) then
		if(v_CA_v_n .ge. -60. .and. v_CA_v_n .lt. -5. ) then
		         call td_hf1_RID( 6120, p_n_tof, n_reactionID )
		else if(v_CA_v_n .ge. -5. .and. v_CA_v_n .lt. 5. ) then
		         call td_hf1_RID( 6140, p_n_tof, n_reactionID )
		else if(v_CA_v_n .ge. 5. .and. v_CA_v_n .lt. 60. ) then
		         call td_hf1_RID(6160, p_n_tof, n_reactionID )
		endif
	   else if(MS_proton_PID) then
	      if(v_trg_v_n .gt. 0.) then 
		         call td_hf1_RID( 6180, p_n_tof, n_reactionID )
		else 
		         call td_hf1_RID( 6200, p_n_tof, n_reactionID )
		endif
	   endif



c ... neutron with proton trigger with n_efficiency
	   if((proton_PID) .or. (deuteron_PID)) then 
		if(norm_dL_thin .lt. 4.) then
		   call td_hf1( 4301, p_trg, 1.)
		else
		   call td_hf1( 4302, p_trg, 1.)
		endif
		call td_hf1_RID( 4320, p_n_tof, n_reactionID )
	      if(v_trg_v_n .lt. 0.) then 
		   call td_hf1_RID( 4340, p_n_tof, n_reactionID )
		   if(norm_dL_thin .lt. 4.) then
		      call td_hf1_RID( 4380, p_n_tof, n_reactionID )
		   else
		      call td_hf1_RID( 4400, p_n_tof, n_reactionID )
		   endif
	      else
		   call td_hf1_RID( 4360, p_n_tof, n_reactionID )
		   if(norm_dL_thin .lt. 4.) then
		      call td_hf1_RID( 4420, p_n_tof, n_reactionID )
		   else
		      call td_hf1_RID( 4440, p_n_tof, n_reactionID )
		   endif
	      endif
	   endif

	   if(deuteron_PID) then 
		call td_hf1_RID( 4460, p_n_tof, n_reactionID )
	      if(v_trg_v_n .lt. -0.4 .and. v_trg_v_n.gt.-0.8) then 
		   call td_hf1_RID( 4480, p_n_tof, n_reactionID )
	      else
		   call td_hf1_RID( 4500, p_n_tof, n_reactionID )
	      endif
	   endif

10	continue
	EndDo
      return
      end



	subroutine neutron_kin(kp_cross, point_n, time_n, tc, 
     &  		tof, n_flight_Length, beta_inv, T_n, p_n, B_KN, v_n)
	implicit none
	real tc
	real kp_cross(3), point_n(3), time_n, T_n, p_n, B_KN
	real rn, n_flight_Length, tof, v_n(3), beta_inv
	real*8 beta, g
c common
      common /masses/ M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
      real  M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
      real  M_helium4, M_helium3, M_triton, M_deuteron, M_nn
      common /masses2/  M_helium4, M_helium3, M_triton, M_deuteron, M_nn
c ... common /PIDs/
      integer PID_pion, PID_pi0, PID_proton, PID_neutron,
     &	PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
      common /PIDs/ PID_pion, PID_pi0, PID_proton, PID_neutron, 
     &      PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
c ... common /setup/ for simulation run-condition, counter resolution and its efficiency
	integer FF_knucl_decay, multi_set, k_decay_set
	logical Kstop_cor
      real  delta_x, delta_y, delta_T0, delta_NC, TC_eff, T0_eff
      common /setup/ delta_x, delta_y, delta_T0, delta_NC, TC_eff, T0_eff,
     &	FF_knucl_decay, multi_set, k_decay_set, Kstop_cor
c external
	real gauss, dT_cor
c ... common /kaon/ for kaon trajectory
      real  kaon_point(3) ! position of the K-reaction
      real  kt_vector(3) ! trajectory of the kaon injection
      real  kt_point(3) ! position of the K-reaction
      real  K_stop_time, K_flight_time, dL_T0
      common /kaon/ kaon_point, kt_point, kt_vector, K_stop_time, K_flight_time, dL_T0
c
	   v_n(1) = ( point_n(1) - kp_cross(1) ) + delta_x*(rn()-0.5)
	   v_n(2) = ( point_n(2) - kp_cross(2) ) + gauss(0., delta_y)
	   v_n(3) = ( point_n(3) - kp_cross(3) ) + 20.*(rn()-0.5)
	   n_flight_Length = sqrt( v_n(1)**2 + v_n(2)**2 + v_n(3)**2 ) 
	   v_n(1) = v_n(1) / n_flight_Length
	   v_n(2) = v_n(2) / n_flight_Length
	   v_n(3) = v_n(3) / n_flight_Length
         tof = time_n + gauss(0., delta_T0) + gauss(0., delta_NC)
	   if(Kstop_cor) tof = tof + 
     &	K_flight_time - dT_cor(kp_cross, tc)
         beta = n_flight_Length / 29.9792458 / tof
	   beta_inv = 1. / beta
	   g = 1.d0 - beta**2
	   if(g .gt. 0.d0) then
            T_n = M_n *( 1.d0 - sqrt(g) ) / sqrt(g) 
         else
            T_n = 400.
         endif
         p_n = sqrt( T_n * ( T_n + 2. * M_n ) )
         B_KN =  sqrt( (M_helium4 + M_K - M_n - T_n)**2 - p_n**2 )
     &	   - M_helium3 - M_K
cxxxx
c	write(*,*) '*', time_n, p_n, n_flight_Length, K_flight_time, dT_cor(kp_cross, tc)
	return
	end



	subroutine proton_kin_ncv(kp_cross, point_p, time_p, tc, 
     &  		beta_inv, T_p, p_p, PID)
	implicit none
	real tc
	real kp_cross(3), point_p(3), time_p, T_p, p_p
	real rn, tof, beta_inv
	real*8 beta
c common
      common /masses/ M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
      real  M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
      real  M_helium4, M_helium3, M_triton, M_deuteron, M_nn
      common /masses2/  M_helium4, M_helium3, M_triton, M_deuteron, M_nn
c ... common /PIDs/
      integer PID_pion, PID_pi0, PID_proton, PID_neutron,
     &	PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
      common /PIDs/ PID_pion, PID_pi0, PID_proton, PID_neutron, 
     &      PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
c ... common /setup/ for simulation run-condition, counter resolution and its efficiency
	integer FF_knucl_decay, multi_set, k_decay_set
	logical Kstop_cor
      real  delta_x, delta_y, delta_T0, delta_NC, TC_eff, T0_eff
      common /setup/ delta_x, delta_y, delta_T0, delta_NC, TC_eff, T0_eff,
     &	FF_knucl_decay, multi_set, k_decay_set, Kstop_cor
c external
	real gauss, dT_cor
c ... common /kaon/ for kaon trajectory
      real  kaon_point(3) ! position of the K-reaction
      real  kt_vector(3) ! trajectory of the kaon injection
      real  kt_point(3) ! position of the K-reaction
      real  K_stop_time, K_flight_time, dL_T0
      common /kaon/ kaon_point, kt_point, kt_vector, K_stop_time, K_flight_time, dL_T0
c
	real point_ps(3), p_flight_Length, v_p(3)
	integer PID
c
	   point_ps(1) =  point_p(1)
	   point_ps(2) =  point_p(2) + gauss(0., delta_y)
	   point_ps(3) =  point_p(3) + 40.*(rn()-0.5)
         tof = time_p + gauss(0., delta_T0) + 2. * gauss(0., delta_NC)
	   if(Kstop_cor) tof = tof + 
     &	K_flight_time - dT_cor(kp_cross, tc)
	      v_p(1) = ( point_ps(1) - kp_cross(1) ) 
	      v_p(2) = ( point_ps(2) - kp_cross(2) ) 
	      v_p(3) = ( point_ps(3) - kp_cross(3) ) 
	      p_flight_Length = sqrt( v_p(1)**2 + v_p(2)**2 + v_p(3)**2 ) 
	      v_p(1) = v_p(1) / p_flight_Length
	      v_p(2) = v_p(2) / p_flight_Length
	      v_p(3) = v_p(3) / p_flight_Length
            beta = p_flight_Length / 29.9792458 / tof
	      beta_inv = 1. / beta
	      call proton_kin_ncv_cor( kp_cross, point_ps, tof, p_p )
            T_p = sqrt(p_p**2 + M_p**2) - M_p
	return
	end



	subroutine proton_kin_ncv_cor( kp_cross, point_p, time_p, p_p )
	implicit none
	real kp_cross(3), point_p(3), time_p, p_p, v_p(3)
	real p_flight_Length, T_p
	real M_p /938.272/
	integer proton /7/
	real p_guess1, p_guess2, t_guess1, t_guess2, t_err1, t_err2
	real point_p_try(3), p_new, p_guess1s, p_guess2s
	integer init, react, decay, multi, fluct, box, status1, status2
	real*8 beta
	integer j
c common
	integer n_start, n_end
	common /ncount/ n_start, n_end
	real p_guess0, p_p_org
	common /pg0/ p_guess0, p_p_org
	integer b_start, b_end
	common /boundary/ b_start, b_end
c
	integer ncv_start, ncv_end
	common /ncv/ ncv_start, ncv_end
c
	integer hit, PID
	real dE, dL, pnt_lcl(3), v_lcl(3), p_lcl
c
	j =0
	   v_p(1) = ( point_p(1) - kp_cross(1) ) 
	   v_p(2) = ( point_p(2) - kp_cross(2) ) 
	   v_p(3) = ( point_p(3) - kp_cross(3) ) 
	   p_flight_Length = sqrt( v_p(1)**2 + v_p(2)**2 + v_p(3)**2 ) 
	   v_p(1) = v_p(1) / p_flight_Length
	   v_p(2) = v_p(2) / p_flight_Length
	   v_p(3) = v_p(3) / p_flight_Length
         beta = p_flight_Length / 29.9792458 / time_p
         T_p = M_p *( 1.d0 - sqrt(1.d0 - beta**2) ) / sqrt(1.d0 - beta**2) 
         p_p = sqrt( T_p * ( T_p + 2. * M_p ) )
	   p_guess0 = p_p
	   p_guess1 = p_p
	   p_guess1s = p_guess1
	   p_guess2 = p_p + 50.
	   p_guess2s = p_guess2
         t_guess1 = 0. 
	   init  = 1
	   react = 0
	   decay = 0
	   multi = 0
	   fluct = 0
	   point_p_try(1) = kp_cross(1)
	   point_p_try(2) = kp_cross(2)
	   point_p_try(3) = kp_cross(3)
cxcx
c	write(*, *) 'point_p_try', point_p_try
cxcx
	   call Travel(p_guess1,point_p_try,v_p,proton,Box,t_guess1,status1,
     &               init, react, decay, multi, fluct)
	   if(Box .ge. b_start .and. Box .le. b_end) go to 30
	   if(v_p(1).gt.0.) then
         	   call object_information(ncv_start, hit, dE, dL, 
     &			pnt_lcl, v_lcl, p_lcl, t_guess1, PID)
	   else
         	   call object_information(ncv_end, hit, dE, dL, 
     &			pnt_lcl, v_lcl, p_lcl, t_guess1, PID)
	   endif
         t_err1 = t_guess1 - time_p 
10	continue
	   j = j + 1
	   init  = 0
         t_guess2 = 0. 
	   p_guess2s = p_guess2
	   point_p_try(1) = kp_cross(1)
	   point_p_try(2) = kp_cross(2)
	   point_p_try(3) = kp_cross(3)
	   call Travel(p_guess2,point_p_try,v_p,proton,Box,t_guess2,status2,
     &               init, react, decay, multi, fluct)
	   if(v_p(1).gt.0.) then
         	   call object_information(ncv_start, hit, dE, dL, 
     &			pnt_lcl, v_lcl, p_lcl, t_guess2, PID)
	   else
         	   call object_information(ncv_end, hit, dE, dL, 
     &			pnt_lcl, v_lcl, p_lcl, t_guess2, PID)
	   endif
c	   if(Box .ne. n_start .and. Box .ne. n_end) go to 30
         t_err2 = t_guess2 - time_p 
	   p_new = (t_err2*p_guess1s - t_err1*p_guess2s) / (t_err2 - t_err1)
cxcx
c	if(p_p_org .lt. 390.) then
c	write(*, *) j, p_guess1s, p_guess2s,t_err1,t_err2, status1,status2
c     &	, p_new
c	endif
cxcx
	   if(abs(p_new-p_guess1s) .lt. 1.) go to 20
	   if(abs(p_new-p_guess2s) .lt. abs(p_new-p_guess1s)) then
		p_guess1s = p_guess2s
		t_err1 = t_err2
		p_guess2 = p_new
	      if(j.gt.10) go to 25
		go to 10
	   else
		p_guess2 = p_new
	      if(j.gt.10) go to 25
		go to 10
	   endif
20	continue
	   if(Box .ge. b_start .and. Box .le. b_end) go to 30
	   p_p = p_new
	return
25	continue
	   p_p = p_new + 10000.
	return
30	continue
	   p_p = -10000.
	return
	end



	subroutine proton_kin(kp_cross, point_p, time_p, tc, 
     &  		beta_inv, T_p, p_p, PID)
	implicit none
	real tc
	real kp_cross(3), point_p(3), time_p, T_p, p_p
	real rn, tof, beta_inv
	real*8 beta
c common
      common /masses/ M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
      real  M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
      real  M_helium4, M_helium3, M_triton, M_deuteron, M_nn
      common /masses2/  M_helium4, M_helium3, M_triton, M_deuteron, M_nn
c ... common /PIDs/
      integer PID_pion, PID_pi0, PID_proton, PID_neutron,
     &	PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
      common /PIDs/ PID_pion, PID_pi0, PID_proton, PID_neutron, 
     &      PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
c ... common /setup/ for simulation run-condition, counter resolution and its efficiency
	integer FF_knucl_decay, multi_set, k_decay_set
	logical Kstop_cor
      real  delta_x, delta_y, delta_T0, delta_NC, TC_eff, T0_eff
      common /setup/ delta_x, delta_y, delta_T0, delta_NC, TC_eff, T0_eff,
     &	FF_knucl_decay, multi_set, k_decay_set, Kstop_cor
c external
	real gauss, dT_cor
c ... common /kaon/ for kaon trajectory
      real  kaon_point(3) ! position of the K-reaction
      real  kt_vector(3) ! trajectory of the kaon injection
      real  kt_point(3) ! position of the K-reaction
      real  K_stop_time, K_flight_time, dL_T0
      common /kaon/ kaon_point, kt_point, kt_vector, K_stop_time, K_flight_time, dL_T0
c
	real point_ps(3), p_flight_Length, v_p(3)
	integer PID
c
	   point_ps(1) =  point_p(1)
	   point_ps(2) =  point_p(2) + gauss(0., delta_y)
	   point_ps(3) =  point_p(3) + 20.*(rn()-0.5)
         tof = time_p + gauss(0., delta_T0) + gauss(0., delta_NC)
	   if(Kstop_cor) tof = tof + 
     &	K_flight_time - dT_cor(kp_cross, tc)
	      v_p(1) = ( point_ps(1) - kp_cross(1) ) 
	      v_p(2) = ( point_ps(2) - kp_cross(2) ) 
	      v_p(3) = ( point_ps(3) - kp_cross(3) ) 
	      p_flight_Length = sqrt( v_p(1)**2 + v_p(2)**2 + v_p(3)**2 ) 
	      v_p(1) = v_p(1) / p_flight_Length
	      v_p(2) = v_p(2) / p_flight_Length
	      v_p(3) = v_p(3) / p_flight_Length
            beta = p_flight_Length / 29.9792458 / tof
	      beta_inv = 1. / beta
	   if(PID .eq. PID_proton) then
	      call proton_kin_cor( kp_cross, point_ps, tof, p_p )
            T_p = sqrt(p_p**2 + M_p**2) - M_p
	   else if(abs(PID) .eq. PID_pion) then
            beta = p_flight_Length / 29.9792458 / tof
	      beta_inv = 1. / beta
            T_p = M_n *( 1.d0 - sqrt(1.d0 - beta**2) ) / sqrt(1.d0 - beta**2) 
            p_p = sqrt( T_p * ( T_p + 2. * M_n ) )
	   else 
            p_p = 0.
	   endif
	return
	end




	subroutine proton_kin_cor( kp_cross, point_p, time_p, p_p )
	implicit none
	real kp_cross(3), point_p(3), time_p, p_p, v_p(3)
	real p_flight_Length, T_p
	real M_p /938.272/
	integer proton /7/
	real p_guess1, p_guess2, t_guess1, t_guess2, t_err1, t_err2
	real point_p_try(3), p_new, p_guess1s, p_guess2s
	integer init, react, decay, multi, fluct, box, status1, status2
	real*8 beta
	integer j
c common
	integer n_start, n_end
	common /ncount/ n_start, n_end
	real p_guess0, p_p_org
	common /pg0/ p_guess0, p_p_org
	integer b_start, b_end
	common /boundary/ b_start, b_end
c
	j =0
	   v_p(1) = ( point_p(1) - kp_cross(1) ) 
	   v_p(2) = ( point_p(2) - kp_cross(2) ) 
	   v_p(3) = ( point_p(3) - kp_cross(3) ) 
	   p_flight_Length = sqrt( v_p(1)**2 + v_p(2)**2 + v_p(3)**2 ) 
	   v_p(1) = v_p(1) / p_flight_Length
	   v_p(2) = v_p(2) / p_flight_Length
	   v_p(3) = v_p(3) / p_flight_Length
         beta = p_flight_Length / 29.9792458 / time_p
         T_p = M_p *( 1.d0 - sqrt(1.d0 - beta**2) ) / sqrt(1.d0 - beta**2) 
         p_p = sqrt( T_p * ( T_p + 2. * M_p ) )
	   p_guess0 = p_p
	   p_guess1 = p_p
	   p_guess1s = p_guess1
	   p_guess2 = p_p + 50.
	   p_guess2s = p_guess2
         t_guess1 = 0. 
	   init  = 1
	   react = 0
	   decay = 0
	   multi = 0
	   fluct = 0
	   point_p_try(1) = kp_cross(1)
	   point_p_try(2) = kp_cross(2)
	   point_p_try(3) = kp_cross(3)
cxcx
c	write(*, *) 'point_p_try', point_p_try
cxcx
	   call Travel(p_guess1,point_p_try,v_p,proton,Box,t_guess1,status1,
     &               init, react, decay, multi, fluct)
	   if(Box .ge. b_start .and. Box .le. b_end) go to 30
         t_err1 = t_guess1 - time_p 
10	continue
	   j = j + 1
	   init  = 0
         t_guess2 = 0. 
	   p_guess2s = p_guess2
	   point_p_try(1) = kp_cross(1)
	   point_p_try(2) = kp_cross(2)
	   point_p_try(3) = kp_cross(3)
	   call Travel(p_guess2,point_p_try,v_p,proton,Box,t_guess2,status2,
     &               init, react, decay, multi, fluct)
c	   if(Box .ne. n_start .and. Box .ne. n_end) go to 30
         t_err2 = t_guess2 - time_p 
	   p_new = (t_err2*p_guess1s - t_err1*p_guess2s) / (t_err2 - t_err1)
cxcx
c	if(p_p_org .lt. 390.) then
c	write(*, *) j, p_guess1s, p_guess2s,t_err1,t_err2, status1,status2
c     &	, p_new
c	endif
cxcx
	   if(abs(p_new-p_guess1s) .lt. 1.) go to 20
	   if(abs(p_new-p_guess2s) .lt. abs(p_new-p_guess1s)) then
		p_guess1s = p_guess2s
		t_err1 = t_err2
		p_guess2 = p_new
	      if(j.gt.10) go to 25
		go to 10
	   else
		p_guess2 = p_new
	      if(j.gt.10) go to 25
		go to 10
	   endif
20	continue
	   if(Box .ge. b_start .and. Box .le. b_end) go to 30
	   p_p = p_new
	return
25	continue
	   p_p = p_new + 10000.
	return
30	continue
	   p_p = -10000.
	return
	end




	real function dT_cor(kp_cross, tc)
	implicit none 
	real kp_cross(3), tc
	real t, zk
	zk = (kp_cross(3) - tc) * 10.
	t =  (zk + 75.) / 150. 
c	if(t .gt. 0.01 .and. t .lt. 0.99 ) then
	  dT_cor = 3.16 + 0.00666 * zk
c	else
c	  dT_cor = -10000.
c	endif

c	zk = (kp_cross(3) - target_center) * 10.
c	t = - (zk - 40.) / 80. 
c	if(t .gt. 0.01 .and. t .lt. 0.99 ) then
c	  dT_cor = 3.76 + 0.41 * 
c     &	( (1./(1-t**2)-1.) / (1./(1-0.75**2)-1.) )**(1.5)
c	  if(dT_cor.gt. 6.) dT_cor = 6.
c	else
c	  dT_cor = -1000.
c	endif
	return
	end




	subroutine pion_kin( kp_cross, point_pi, time_pi, delta_x, delta_y, delta_T0,
     &  		T_pi, p_pi, v_pi)
	implicit none
	real kp_cross(3), point_pi(3), time_pi, T_pi, p_pi, delta_x, delta_y
	real pi_flight_Length, tof, beta, delta_T0, v_pi(3)
c common
      common /masses/ M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
      real  M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
      real  M_helium4, M_helium3, M_triton, M_deuteron, M_nn
      common /masses2/  M_helium4, M_helium3, M_triton, M_deuteron, M_nn
c ... common /PIDs/
      integer PID_pion, PID_pi0, PID_proton, PID_neutron,
     &	PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
      common /PIDs/ PID_pion, PID_pi0, PID_proton, PID_neutron, 
     &      PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
c
	common /trg/ target_center
	real target_center
c external
	real gauss
c
	   pi_flight_Length = sqrt( ( point_pi(1) - kp_cross(1) )**2
     &			  + ( point_pi(2) - kp_cross(2) )**2
     &			  + ( point_pi(3) - kp_cross(3) )**2 ) 
	   v_pi(1) = ( point_pi(1) - kp_cross(1) ) / pi_flight_Length
	   v_pi(2) = ( point_pi(2) - kp_cross(2) ) / pi_flight_Length
	   v_pi(3) = ( point_pi(3) - kp_cross(3) ) / pi_flight_Length
         tof = time_pi + gauss(0., 0.15)
         beta = pi_flight_Length / 29.9792458 / tof
c	write(*,*) 'pi_flight_Length in pion_kin', pi_flight_Length, time_pi, beta
         T_pi = M_pi *( 1.d0 - sqrt(1.d0 - beta**2) ) / sqrt(1.d0 - beta**2) 
         p_pi = sqrt( T_pi * ( T_pi + 2. * M_pi ) )
	   return
	end



c +++ utility +++++++++++++++++++++++++++++++++++++++++++++++


      real function Kinetic_Energy( mass, momentum )
      implicit NONE
      real  mass, momentum
       if(momentum/mass .lt. 0.05) then   ! best switching value for real*4
         Kinetic_Energy = (1.-(momentum/Mass)**2/4.)*momentum**2/Mass/2.
       else
         Kinetic_Energy = sqrt(Mass**2 + momentum**2) - Mass
       endif
      return
      end



	real function inv_mass(m1, p1, v1, m2, p2, v2)
	implicit NONE
	real m1, p1, v1(3), m2, p2, v2(3)
c local
	real E1, E2, Kinetic_Energy
c begin
	E1 = m1 + Kinetic_Energy(m1, p1)
	E2 = m2 + Kinetic_Energy(m2, p2)
	inv_mass = (E1 + E2)**2 
     &	- (p1*v1(1) + p2*v2(1))**2
     &	- (p1*v1(2) + p2*v2(2))**2
     &	- (p1*v1(3) + p2*v2(3))**2
	if(inv_mass .gt. 0. ) then
	   inv_mass = sqrt(inv_mass)
	else
	   inv_mass = -1000.
	endif
	return
	end



      real  function vertex_diff(x, u, y, v, cross, diff) 
      implicit NONE
      real  x(3), y(3), u(3), v(3), xy_2, cross(3)
      real  s, t, xy(3), uv_p, uv(3), vu(3), diff(3)
      real  sprod
      integer j
c begin
      uv_p  = sprod( u, v )
c      if(ev_NUM .eq. ev_error) write(*,*) 'uv_p', uv_p, 1. - uv_p, 1. - uv_p**2
      if( abs(uv_p) .lt. 1.-1.e-5) then
         do j = 1, 3
             xy(j) = x(j) - y(j)
             uv(j) = uv_p * v(j) - u(j)
             vu(j) = v(j) - uv_p * u(j)
         enddo
         s  = sprod( uv, xy ) / (1. - uv_p**2)
         t  = sprod( vu, xy ) / (1. - uv_p**2)
         do j = 1, 3
             diff(j) = -((x(j) + s*u(j)) - (y(j) + t*v(j)))
c             cross(j) = ((x(j) + s*u(j)) + (y(j) + t*v(j))) / 2.
             cross(j) = x(j) + s*u(j)
         enddo
c?????
c	write(*,*) '*', (x(1) + s*u(1)), (x(2) + s*u(2)), (x(3) + s*u(3)),
c     &	(y(1) + t*v(1)), (y(2) + t*v(2)), (y(3) + t*v(3))
         vertex_diff =  sqrt(diff(1)**2 + diff(2)**2 + diff(3)**2)
      else  
         do j = 1, 3
            xy(j) = x(j) - y(j)
		cross(j) = 100000000.
         enddo
         xy_2 = xy(1)**2 + xy(2)**2 + xy(3)**2
c	write(*,*) 'here? xy_2', xy_2, sprod( u, xy )**2, sprod( u, xy )
c	write(*,*) 'here? ', xy_2 - sprod( u, xy )**2
         vertex_diff =  sqrt( abs(xy_2 - sprod( u, xy )**2) )
      endif
c	write(*,*) 'vertex_diff',vertex_diff
      return
      end



		
